
# q-expansion of newform 304.2.i.b, downloaded from the LMFDB on 15 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 304
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[191, [1, 0]], [229, [1, 0]], [97, [0, -1]]]
aps_data = [[0, 0], [1, -1], [-3, 3], [0, 0], [4, 0], [0, 5], [5, -5], [-5, 2], [0, -1], [0, -3], [-4, 0], [2, 0], [5, -5], [-11, 11], [0, -5], [0, 9], [13, -13], [0, 1], [0, -5], [1, -1], [9, -9], [17, -17], [-16, 0], [0, -3], [13, -13], [0, -19], [0, 0], [12, 0], [-7, 7], [6, 0], [0, 15], [-15, 15], [0, 5], [0, 15], [17, -17], [-16, 0], [13, -13], [4, 0], [0, -5], [5, -5], [12, 0], [0, -7], [16, 0], [-15, 15], [10, 0], [0, 3], [9, -9], [-11, 11], [-12, 0], [-22, 0], [-11, 11], [-12, 0], [0, 21], [0, -9], [0, 25], [-19, 19], [17, -17], [29, -29], [-30, 0], [0, -15], [-15, 15], [10, 0], [-3, 3], [-12, 0], [0, 5], [0, -27], [-20, 0], [-3, 3], [5, -5], [-14, 0], [-10, 0], [5, -5], [0, -5], [-10, 0], [12, 0], [-15, 15], [0, 17], [-35, 35], [-11, 11], [0, -7], [-12, 0], [29, -29], [0, -9], [0, 25], [-7, 7], [0, 15], [-2, 0], [10, 0], [1, -1], [20, 0], [-12, 0], [0, 27], [40, 0], [9, -9], [-35, 35], [0, 15], [0, 5], [-6, 0], [0, -29], [0, -11], [0, 7], [0, 33], [-4, 0], [-18, 0], [4, 0], [-2, 0], [25, -25], [0, -35], [0, -25], [-34, 0], [0, 0], [1, -1], [0, -3], [20, 0], [0, -29], [9, -9], [5, -5], [0, 0], [-6, 0], [0, 19], [0, 29], [30, 0], [-30, 0], [0, 0], [-36, 0], [-23, 23], [0, 17], [-27, 27], [13, -13], [-6, 0], [37, -37], [29, -29], [0, 31], [-35, 35], [38, 0], [0, -23], [0, 25], [-20, 0], [42, 0], [-42, 0], [0, -21], [0, -11], [0, -5], [0, 15], [30, 0], [-35, 35], [21, -21], [-7, 7], [0, -13], [36, 0], [-47, 47], [14, 0], [0, 55], [0, -45], [-35, 35], [56, 0], [40, 0], [-7, 7], [0, 17], [0, -39], [-35, 35], [-15, 15], [-23, 23], [-3, 3], [30, 0], [25, -25], [-55, 55], [0, 25]]
